import { EventExecutor } from './event/EventExecutor';
export declare class Aggregate {
    static applyEvents(allEvents: Object[], eventExecutors: EventExecutor<any, any>[], aggregateClasses: Map<string, Function>): Aggregate;
    static applyEventOnAggregate(event: Object, eventExecutor: EventExecutor<any, any>, aggregate: Aggregate): void;
    static instantiateAggregate(aggregateType: string, aggregateId: string, aggregateClasses: Map<string, Function>): Aggregate;
    private aggregateRoles;
    aggregateId: string;
    aggregateType: string;
    /**
     * Allow optimistic locking to support aggregate snapshot.
     */
    private maxEventOrdinal;
    constructor(aggregateType: string, aggregateId: string);
    getAggregateType(): string;
    getAggregateId(): string;
    getFullId(): string;
    /**
     * Gets the extra roles that this aggregate grants
     */
    getAggregateRoles(): Map<string, Array<string>>;
    /**
     * Gets the extra roles that this aggregate grants to an entity
     * @param {string} Optional. The id of the entity we're asking about.
     * @returns {[string]} The list of roles for this aggregate
     */
    getAggregateRolesFor(entityId: any): Array<string>;
    addAggregateRole(entityId: string, roles: Array<string>): void;
    removeAggregateRole(entityId: string, roles?: Array<string>): void;
    /**
     * Check that this event can be applied to this aggregate as the ordinal is the next one
     *
     * @param event
     * @returns number
     */
    checkEventCanBeApplied(event: any, eventOrdinal: number, eventId: any): void;
    /**
     * Use event ordinal to check if an event has been applied to the aggregate.
     * @param eventOrdinal
     * @returns boolean
     */
    isPastOrdinal(eventOrdinal: number): boolean;
    /**
     * Use eventCount if its great than 0 or use eventsTotal
     *
     * @returns number
     */
    getNextEventOrdinal(): number;
    getMaxEventOrdinal(): number;
    private setMaxEventOrdinal(maxEventOrdinal);
}
