"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const inceptum_1 = require("inceptum");
const LockViolationError_1 = require("../../error/LockViolationError");
const AggregateEventStore_1 = require("./AggregateEventStore");
exports.mySQLDateTimeFormat = 'YYYY-MM-DD HH:mm:ss';
exports.mySQLDateFormat = 'YYYY-MM-DD';
const logger = inceptum_1.LogManager.getLogger(__filename);
class AbstractMysqlAggregateEventStore extends AggregateEventStore_1.AggregateEventStore {
    getEventsOf(aggregateId) {
        return __awaiter(this, void 0, void 0, function* () {
            const sql = `SELECT *
                    FROM aggregate_events
                    WHERE aggregate_id = ?
                    ORDER BY ordinal, event_committed_time ASC`;
            const rows = yield this.mysqlClient.read(sql, aggregateId);
            return rows.map((row) => {
                return this.dataToAggregateEvent(row['event_content']);
            });
        });
    }
    commitEvent(aggregateEvent) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = this.aggregateEventToData(aggregateEvent);
            const sql = `INSERT INTO aggregate_events (
      event_id,
      aggregate_id,
      event_created_time,
      event_committed_time,
      event_type,
      event_content,
      ordinal)
    VALUES ?`;
            try {
                yield this.mysqlClient.executeSql(false, sql, [data]);
            }
            catch (e) {
                if (this.isLockViolation(e)) {
                    // throw locking violation
                    this.throwLockViolation(e);
                }
                throw e;
            }
        });
    }
    commitAllEvents(aggregateEvents) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = aggregateEvents.map(this.aggregateEventToData);
            const sql = `INSERT INTO aggregate_events (
      event_id,
      aggregate_id,
      event_created_time,
      event_committed_time,
      event_type,
      event_content,
      ordinal)
    VALUES ?`;
            try {
                yield this.mysqlClient.executeSql(false, sql, data);
            }
            catch (e) {
                if (this.isLockViolation(e)) {
                    // throw locking violation
                    this.throwLockViolation(e);
                }
                throw e;
            }
        });
    }
    isLockViolation(e) {
        return /^(ER_DUP_ENTRY).*(ordinal)/.test(e.message);
    }
    throwLockViolation(cause) {
        throw new LockViolationError_1.LockViolationError('State has been changed since the resource was first accessed by the transaction.', cause);
    }
}
__decorate([
    inceptum_1.Autowire('AggregateMySQLClient')
], AbstractMysqlAggregateEventStore.prototype, "mysqlClient", void 0);
exports.AbstractMysqlAggregateEventStore = AbstractMysqlAggregateEventStore;
//# sourceMappingURL=AbstractMysqlAggregateEventStore.js.map